# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import time
import uuid
import re
import socket
from .utils import Request, Log
from .service import Message
from .__config__ import __apiUrl__, __messageService__, __version__


class Init:

    def __init__(self, app_id: str = "", app_secret: str = ""):

        Log.info("RoboMentor_Client " + __version__)

        assert app_id != "" or app_secret != "", "app_id OR app_secret Error"

        auth_time = time.strftime('%Y%m%d%H%M', time.localtime(time.time()))
        self.auth_ip = self.get_host_ip()
        self.auth_mac = self.get_mac_address()
        self.app_id = app_id
        self.app_secret = app_secret

        headers = {
            "Content-Type": "application/json",
            "Robot-Token": app_id + "@" + app_secret + "@" + auth_time
        }

        params = {"app_id": app_id, "app_secret": app_secret, "robot_mac": self.auth_mac, "robot_ip": self.auth_ip}

        res = Request.do(__apiUrl__ + "/oauth/robot/register", params, headers, 'GET')
        res_json = res.json()

        assert res_json["code"] == 0, "robot register Error"

        self.token = res_json["data"]["token"]
        self.robot_title = res_json["data"]["robot_title"]
        self.robot_net_ip = res_json["data"]["robot_net_ip"]

        self.message = Message(__messageService__, self.auth_mac, app_id, app_secret).start()

    @staticmethod
    def get_host_ip():
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        finally:
            s.close()
        return ip

    @staticmethod
    def get_mac_address():
        return ":".join(re.findall(r".{2}", uuid.uuid1().hex[-12:]))



# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import serial
from ..utils import Log


class Serial:

    def __init__(self, port, baudrate, stopbits, timeout: float = 0.2):
        self.port = port
        self.baudrate = baudrate
        self.stopbits = stopbits
        self.timeout = timeout
        self.conn = None
        self.readData = ""
        serial.Serial()

    def open(self):

        ret = False

        try:
            self.conn = serial.Serial(self.port, self.baudrate, self.timeout)
            if self.conn.is_open:
                ret = True
        except Exception as e:
            Log.error(e)
        return self.conn, ret

    def close(self):
        self.conn.close()




# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

__version__ = "1.1.3"
__apiUrl__ = "https://api.robomentor.cn"
__messageService__ = "47.111.96.253"



# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import requests
from .log import Log


class Request:

    @staticmethod
    def do(url, params, headers, http_method):
        res = ''
        if http_method.upper() == "POST":
            try:
                res = requests.post(url, params, headers=headers)
                Log.info("正在发送POST请求")
            except Exception as e:
                Log.error("POST请求出现了异常：{0}".format(e))
        elif http_method.upper() == "GET":
            try:
                res = requests.get(url, params, headers=headers)
                Log.info("正在发送GET请求")
            except Exception as e:
                Log.error("GET请求出现了异常：{0}".format(e))

        Log.info("收到请求返回数据:")
        Log.info(res.json())

        return res



# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import paho.mqtt.client as mqtt
from queue import Queue


class Message:

    def __init__(self, host, client_id, username, password):
        self.host = host
        self.port = 1883
        self.client_id = client_id
        self.username = username
        self.password = password
        self.mqtt_client = None
        self.timeout = 60
        self.message = Queue(10)

    def on_connect(self, client, userdata, flags, rc):
        self.subscribe("robot/" + self.client_id)

    def on_message(self, client, userdata, msg):
        self.message.put(str(msg.payload, encoding="utf-8"))

    def get_message(self):
        data = None
        if not self.message.empty():
            data = self.message.get()
        return data

    def subscribe(self, topic):
        self.mqtt_client.subscribe(topic, 0)

    def publish(self, topic, blob):
        self.mqtt_client.publish(topic, blob)

    def start(self):
        if self.mqtt_client is None:
            self.mqtt_client = mqtt.Client("robot-" + self.client_id)
            self.mqtt_client.username_pw_set(self.username, self.password)
            self.mqtt_client.on_connect = self.on_connect
            self.mqtt_client.on_message = self.on_message
            self.mqtt_client.connect(self.host, self.port, self.timeout)
            self.mqtt_client.loop_start()

        return self

    def stop(self):
        if self.mqtt_client is not None:
            self.mqtt_client.loop_stop()
            self.mqtt_client.disconnect()
            self.mqtt_client = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import io
import os
import sys
from shutil import rmtree
from setuptools import find_packages, setup, Command

Dir = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(Dir, "robomentor_client", '__config__.py')) as f:
    exec(f.read(), about)

version = about["__version__"]

try:
    with io.open(os.path.join(Dir, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = ''


class UploadCommand(Command):

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(Dir, 'dist'))
        except OSError:
            pass

        self.status('Preparing {0} v{1}...' . format('RoboMentor_Client', version))

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        sys.exit()


setup(
    name='RoboMentor_Client',
    version=version,
    description='Python library and framework for robomentor_client.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Super Yang',
    author_email='admin@wileho.com',
    python_requires='>=3.6.0',
    url='https://github.com/robomentor/RoboMentor_Client',
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    install_requires=[
        'opencv-contrib-python',
        'paho-mqtt',
        'pyserial',
    ],
    extras_require={
        'python 3.6.x needs': ['dataclasses'],
    },
    include_package_data=True,
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    cmdclass={
        'upload': UploadCommand,
    }
)
